import collections
import numpy as np

from libmultilabel import linear


def calculate_unrealistic_prediction(preds, target):
    nr_nonzero = np.count_nonzero(target, axis=1)
    unreal_preds = np.zeros(preds.shape)
    for i, pred in enumerate(preds):
        top_k_idx = np.argpartition(pred, -nr_nonzero[i])[-nr_nonzero[i] :]
        unreal_preds[i][top_k_idx] = 1
    return unreal_preds


def main():
    dataset_names = ["flickr", "blogcatalog"]
    num_splits = 5
    liblinear_options = "-s 0 -e 0.0001"
    monitor_metrics = ["Macro-F1", "Micro-F1"]

    all_metrics = collections.defaultdict(float)
    for name in dataset_names:
        data_dir = f"data/deepwalk/{name}"

        for i in range(num_splits):
            # Prepare datasets
            datasets = linear.load_dataset("svm", f"{data_dir}/train{i}.svm", f"{data_dir}/test{i}.svm")
            preprocessor = linear.Preprocessor()
            preprocessor.fit(datasets)
            datasets = preprocessor.transform(datasets)
            target = datasets["test"]["y"].toarray()

            # Binary relevance
            model = linear.train_1vsrest(datasets["train"]["y"], datasets["train"]["x"], options=liblinear_options)
            preds = linear.predict_values(model, datasets["test"]["x"])
            binary_relevance_metrics = linear.compute_metrics(preds, target, monitor_metrics)

            # Unrealistic
            preds = calculate_unrealistic_prediction(preds, target)
            unrealistic_metrics = linear.compute_metrics(preds, target, monitor_metrics)

            # Thresholding
            model = linear.train_thresholding(datasets["train"]["y"], datasets["train"]["x"], options=liblinear_options)
            preds = linear.predict_values(model, datasets["test"]["x"])
            thresholding_metrics = linear.compute_metrics(preds, target, monitor_metrics)

            for metric in monitor_metrics:
                all_metrics[f"{name}_binaryrelevance_{metric}"] += binary_relevance_metrics[metric]
                all_metrics[f"{name}_unrealistic_{metric}"] += unrealistic_metrics[metric]
                all_metrics[f"{name}_thresholding_{metric}"] += thresholding_metrics[metric]

    # Average the scores.
    all_metrics = {k: score / num_splits for k, score in all_metrics.items()}

    # Convert to latex
    latex_str = ""
    for method in ["Unrealistic", "Binary relevance", "Thresholding"]:
        latex_str += f"{method: <16}"
        method = method.lower().replace(" ", "")
        for name in dataset_names:
            for metric in ["Macro-F1", "Micro-F1"]:
                key = f"{name}_{method}_{metric}"
                latex_str += f"  & {all_metrics[key]:.3f}"
        latex_str += " \\\\\n"
        if method == "unrealistic":
            latex_str += "\\hline \n"

    print(latex_str)


main()
